define([ "jquery", "base1/cockpit", "shell/shell" ], function($, cockpit, shell) {
    shell.esc = function esc(str) {
        if (str === null || str === undefined) return "";
        var pre = document.createElement("pre");
        var text = document.createTextNode(str);
        pre.appendChild(text);
        return pre.innerHTML.replace(/"/g, "&quot;").replace(/'/g, "&#39;");
    };
    shell.format_delay = function format_delay(d) {
        var seconds = Math.round(d / 1e3);
        var minutes = Math.floor(seconds / 60);
        var hours = Math.floor(minutes / 60);
        seconds = seconds - minutes * 60;
        minutes = minutes - hours * 60;
        var s = seconds + " seconds";
        if (minutes > 0) s = minutes + " minutes, " + s;
        if (hours > 0) s = hours + " hours, " + s;
        return s;
    };
    shell.find_in_array = function find_in_array(array, elt) {
        for (var i = 0; i < array.length; i++) {
            if (array[i] == elt) return true;
        }
        return false;
    };
    shell.action_btn = function action_btn(func, spec, btn_classes) {
        var direct_btn, indirect_btns, btn;
        var direct_action, disabled;
        direct_btn = $("<button>", {
            "data-container": "body",
            "class": "btn btn-default"
        }).text("").addClass(btn_classes);
        indirect_btns = [];
        disabled = [];
        spec.forEach(function(s, i) {
            indirect_btns[i] = $("<li>", {
                "class": "presentation"
            }).append($("<a>", {
                role: "menuitem",
                on: {
                    click: function(e) {
                        if (!disabled[i] && !$(e.currentTarget).hasClass("disabled")) func(s.action);
                    }
                }
            }).addClass(btn_classes).addClass(s.btn_classes).append($("<span>", {
                "class": s.danger ? "text-danger" : ""
            }).text(s.title)));
            disabled[i] = false;
        });
        btn = $("<div>", {
            "class": "btn-group"
        }).append(direct_btn, $("<button>", {
            "class": "btn btn-default dropdown-toggle",
            "data-toggle": "dropdown"
        }).append($("<span>", {
            "class": "caret"
        })), $("<ul>", {
            "class": "dropdown-menu",
            style: "right:0px;left:auto;min-width:0;text-align:left",
            role: "menu"
        }).append(indirect_btns));
        function select(a) {
            spec.forEach(function(s, i) {
                if (s.action == a || a == "default" && s.is_default) {
                    direct_action = s.action;
                    direct_btn.text(s.title);
                    direct_btn.toggleClass("btn-danger", s.danger);
                    direct_btn.toggleClass("btn-default", !s.danger);
                    direct_btn.off("click");
                    direct_btn.on("click", function() {
                        func(s.action);
                    });
                    direct_btn.prop("disabled", disabled[i]);
                }
            });
        }
        function enable(a, val) {
            if (direct_action == a) direct_btn.prop("disabled", !val);
            spec.forEach(function(s, i) {
                if (s.action == a) {
                    disabled[i] = !val;
                    indirect_btns[i].toggleClass("disabled", !val);
                }
            });
        }
        select("default");
        $.data(btn[0], "cockpit-action-btn-funcs", {
            select: select,
            enable: enable
        });
        return btn;
    };
    shell.action_btn_select = function action_btn_select(btn, action) {
        $.data(btn[0], "cockpit-action-btn-funcs").select(action);
    };
    shell.action_btn_enable = function action_btn_enable(btn, action, val) {
        $.data(btn[0], "cockpit-action-btn-funcs").enable(action, val);
    };
    shell.select_btn = function select_btn(func, spec) {
        var div, btn;
        function option_mapper(opt) {
            if (opt.group && Array.isArray(opt.group)) {
                var group = $("<optgroup>").attr("label", opt.title ? opt.title : "").append(opt.group.map(option_mapper));
                return group;
            } else {
                return $("<option>", {
                    value: opt.choice
                }).text(opt.title);
            }
        }
        btn = $('<select class="form-control">').append(spec.map(option_mapper));
        btn.on("change", function() {
            func(btn.val());
        });
        function select(a) {
            btn.val(a);
            btn.selectpicker("render");
        }
        function selected() {
            return btn.val();
        }
        div = $("<div>").append(btn);
        btn.selectpicker();
        $.data(div[0], "cockpit-select-btn-funcs", {
            select: select,
            selected: selected
        });
        return div;
    };
    shell.select_btn_select = function select_btn_select(btn, choice) {
        $.data(btn[0], "cockpit-select-btn-funcs").select(choice);
    };
    shell.select_btn_selected = function select_btn_selected(btn) {
        return $.data(btn[0], "cockpit-select-btn-funcs").selected();
    };
    shell.util = shell.util || {};
    function cache_debug() {
        if (window.debugging == "all" || window.debugging == "dbus") console.debug.apply(console, arguments);
    }
    shell.util.make_resource_cache = make_resource_cache;
    function make_resource_cache() {
        var resources = {};
        function get(key, create) {
            var handle;
            handle = resources[key];
            if (!handle) {
                cache_debug("Creating %s", key);
                handle = {
                    refcount: 1,
                    resource: create()
                };
                resources[key] = handle;
                handle.resource.release = function() {
                    cache_debug("Releasing %s", key);
                    window.setTimeout(function() {
                        if (!handle.refcount) {
                            console.warn("Releasing unreffed resource");
                        } else {
                            handle.refcount -= 1;
                            if (handle.refcount === 0) {
                                delete resources[key];
                                cache_debug("Closing %s", key);
                                handle.resource.close("unused");
                            }
                        }
                    }, 1e4);
                };
            } else {
                cache_debug("Getting %s", key);
                handle.refcount += 1;
            }
            return handle.resource;
        }
        return {
            get: get
        };
    }
    var machine_info_promises = {};
    shell.util.machine_info = machine_info;
    function machine_info(address) {
        var pr = machine_info_promises[address];
        var dfd;
        if (!pr) {
            dfd = $.Deferred();
            machine_info_promises[address] = pr = dfd.promise();
            cockpit.spawn([ "cat", "/proc/meminfo", "/proc/cpuinfo" ], {
                host: address
            }).done(function(text) {
                var info = {};
                var match = text.match(/MemTotal:[^0-9]*([0-9]+) [kK]B/);
                var total_kb = match && parseInt(match[1], 10);
                if (total_kb) info.memory = total_kb * 1024;
                info.cpus = 0;
                var re = new RegExp("^processor", "gm");
                while (re.test(text)) info.cpus += 1;
                dfd.resolve(info);
            }).fail(function() {
                dfd.reject();
            });
        }
        return pr;
    }
    shell.util.hostname_for_display = hostname_for_display;
    function hostname_for_display(iface) {
        if (iface.PrettyHostname) return iface.PrettyHostname; else if (iface.StaticHostname && iface.StaticHostname != "localhost" && iface.StaticHostname != "localhost.localdomain") return iface.StaticHostname; else if (iface._client && iface._client.target != "localhost") return iface._client.target; else if (iface.client && iface.client.options.host && iface.client.options.host != "localhost") return iface.client.options.host; else return window.location.hostname;
    }
});