define([ "jquery", "base1/cockpit" ], function($, cockpit) {
    "use strict";
    var module = {};
    function reflow_bar_graph(graph, div) {
        var parts;
        if (graph) {
            var selector = "div.bar-row[graph='" + graph + "']";
            parts = $(selector);
        } else if (div) {
            parts = $(div);
        } else {
            parts = $([]);
        }
        function value_parts(el) {
            var value = $(el).attr("value");
            if (value === undefined) return [ NaN ];
            var values = value.split("/", 2);
            var portion = parseInt(values[0], 10);
            if (values.length == 1) return [ portion ];
            var limit = parseInt(values[1], 10);
            if (!isNaN(limit) && portion > limit) portion = limit;
            if (portion < 0) portion = 0;
            return [ portion, limit ];
        }
        var max = 0;
        parts.each(function() {
            var limit = value_parts(this).pop();
            if (!isNaN(limit) && limit > max) max = limit;
        });
        if (graph) {
            var bound = 100 * 1024 * 1024;
            max = max - max % bound + bound;
        }
        parts.each(function() {
            var bits = value_parts(this);
            var portion = bits.shift();
            var limit = bits.pop();
            if (isNaN(portion) || limit === 0) {
                $(this).css("visibility", "hidden");
            } else {
                var bar_progress = $(this).data("bar-progress");
                if (isNaN(limit)) {
                    bar_progress.addClass("progress-no-limit");
                    limit = portion;
                } else {
                    bar_progress.removeClass("progress-no-limit");
                }
                $(this).css("visibility", "visible");
                bar_progress.css("width", limit / max * 100 + "%");
                $(this).data("bar-progress-bar").css("width", portion / limit * 100 + "%").toggle(portion > 0);
            }
        });
    }
    var reflow_timeouts = {};
    function reflow_bar_graph_soon(graph, div) {
        if (graph === undefined) {
            if (div) graph = $(div).attr("graph");
        }
        if (!graph) {
            reflow_bar_graph(undefined, div);
            return;
        }
        if (reflow_timeouts[graph] !== "undefined") window.clearTimeout(reflow_timeouts[graph]);
        reflow_timeouts[graph] = window.setTimeout(function() {
            delete reflow_timeouts[graph];
            reflow_bar_graph(graph);
        }, 10);
    }
    function setup_bar_graph(div) {
        var progress_bar = $("<div>").addClass("progress-bar");
        var progress = $("<div>").addClass("progress").append(progress_bar);
        $(div).addClass("bar-row").append(progress).data("bar-progress", progress).data("bar-progress-bar", progress_bar);
        $(div).attrchange({
            trackValues: false,
            callback: function(event) {
                if (event.attributeName == "graph" || event.attributeName == "value") {
                    reflow_bar_graph_soon(this.getAttribute("graph"), this);
                }
            }
        });
        div.reflow = function() {
            reflow_bar_graph(this.getAttribute("graph"), this);
        };
        reflow_bar_graph_soon(undefined, div);
    }
    function setup_bar_graphs() {
        $("div.bar-row").each(function() {
            setup_bar_graph(this, false);
        });
    }
    module.BarRow = function BarRow(graph) {
        var div = $("<div>").addClass("bar-row").attr("graph", graph);
        setup_bar_graph(div);
        return div;
    };
    $(document).ready(setup_bar_graphs);
    function resize_flex(slider, flex, total, part) {
        var value = 0;
        if (part > total) value = 1; else if (part < 0 || isNaN(part)) value = 0; else if (!isNaN(total) && total > 0 && part >= 0) value = part / total;
        $(flex).css("width", value * 100 + "%").next("div").css("margin-left", $(flex).css("width"));
        slider.value = value;
    }
    function update_value(slider) {
        resize_flex(slider, $(slider).children("div.slider-bar").first()[0], 1, slider.value);
    }
    function check_overflow(slider) {
        $(slider).toggleClass("slider-warning", slider.offsetWidth < slider.scrollWidth);
    }
    function setup_slider(slider) {
        $(slider).attr("unselectable", "on");
        Object.defineProperty(slider, "value", {
            get: function() {
                return parseFloat(this.getAttribute("value"));
            },
            set: function(v) {
                var s = String(v);
                if (s != this.getAttribute("value")) this.setAttribute("value", v);
            }
        });
        Object.defineProperty(slider, "disabled", {
            get: function() {
                if (!this.hasAttribute("disabled")) return false;
                return this.getAttribute("disabled").toLowerCase() != "false";
            },
            set: function(v) {
                this.setAttribute("disabled", v ? "true" : "false");
            }
        });
        update_value(slider);
        check_overflow(slider);
        $(slider).attrchange({
            trackValues: true,
            callback: function(event) {
                if (event.attributeName == "value" && event.oldValue !== event.newValue) update_value(slider);
                if (event.attributeName == "disabled") $(slider).toggleClass("slider-disabled", slider.disabled);
            }
        });
        if (slider.disabled) $(slider).addClass("slider-disabled");
        $(slider).on("mousedown", function(ev) {
            if (slider.disabled) return true;
            var flex;
            var offset = $(slider).offset().left;
            if ($(ev.target).hasClass("slider-thumb")) {
                var hitx = ev.offsetX || ev.clientX - $(ev.target).offset().left;
                offset += hitx - $(ev.target).outerWidth() / 2;
                flex = $(ev.target).parent()[0];
            } else {
                flex = $(slider).children("div.slider-bar").first()[0];
                resize_flex(slider, flex, $(slider).width(), ev.pageX - offset);
                $(slider).trigger("change", [ slider.value ]);
                check_overflow(slider);
            }
            $(document).on("mousemove.slider", function(ev) {
                resize_flex(slider, flex, $(slider).width(), ev.pageX - offset);
                $(slider).trigger("change", [ slider.value ]);
                check_overflow(slider);
                return false;
            }).on("mouseup.slider", function(ev) {
                $(document).off("mousemove.slider").off("mouseup.slider");
                return false;
            });
            return false;
        });
    }
    function setup_sliders() {
        $("div.slider").each(function() {
            setup_slider(this);
        });
    }
    module.Slider = function Slider() {
        var div = $("<div class='slider'>").append($("<div class='slider-bar'>").append($("<div class='slider-thumb'>")));
        setup_slider(div);
        return div;
    };
    $(document).ready(setup_sliders);
    module.OnOff = function OnOff(val, on, off, role_check, btn_classes) {
        function toggle(event) {
            if (role_check && !role_check()) return false;
            box.find(".btn").toggleClass("active");
            box.find(".btn").toggleClass("btn-primary");
            box.find(".btn").toggleClass("btn-default");
            if (on_btn.hasClass("active")) {
                if (off) on(); else on(true);
            } else {
                if (off) off(); else on(false);
            }
            return false;
        }
        var on_btn, off_btn;
        var box = $('<div class="btn-group btn-toggle">').append(on_btn = $('<button class="btn">').text("On").addClass(btn_classes).addClass(!val ? "btn-default" : "btn-primary active").click(toggle), off_btn = $('<button class="btn">').text("Off").addClass(btn_classes).addClass(val ? "btn-default" : "btn-primary active").click(toggle));
        box.set = function set(val) {
            (val ? on_btn : off_btn).addClass("btn-primary active").removeClass("btn-default");
            (val ? off_btn : on_btn).removeClass("btn-primary active").addClass("btn-default");
        };
        box.enable = function enable(val) {
            box.find("button").toggleClass("disabled", !val);
        };
        return box;
    };
    module.update_privileged_ui = function update_privileged_ui(perm, selector, denied_message) {
        var allowed = perm.allowed !== false;
        $(selector).each(function() {
            var allowed_key = "allowed-title";
            if (typeof $(this).data(allowed_key) === "undefined" || $(this).data(allowed_key) === false) $(this).data(allowed_key, $(this).attr("title") || "");
            $(this).tooltip({
                html: true
            });
            if ($(this).hasClass("disabled") === allowed) {
                $(this).toggleClass("disabled", !allowed).attr("data-original-title", null);
                if (allowed) $(this).attr("title", $(this).data(allowed_key)); else $(this).attr("title", denied_message);
                $(this).tooltip("fixTitle");
            }
        });
    };
    return module;
});